//interp.h
/*
 * Copyright (C) 2010 Marko Cebokli   http://lea.hamradio.si/~s57uuu
 * This file is a part of the Frei0r plugin "defish0r"
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


/*******************************************************************
 * The remapping functions use a map aray, which contains a pair
 * of floating values fo each pixel of the output image. These
 * represent the location in the input image, from where the value
 * of the given output pixel should be interpolated.
 * They are given in pixels of the input image.
 * If the output image is wo pixels wide, then the x coordinate
 * of the pixel in row r and column c is at  2*(r*wo+c) in the map
 * array, and y at 2*(r*wo+c)+1
 *
 * The map array is usually computation intensive to generate, and
 * he purpose of the map array is to allow fast remapping of
 * several images (video) using the same map array.
 ******************************************************************/

#include <linux/types.h>

//--------------------------------------------------------
//pointer to an interpolating function
typedef int (*interpp)(unsigned char*, int, int, float, float, unsigned char*);


//**********************************************************
//REMAPPING FUNCTIONS

//------------------------------------------------------------
// remaps an image, based on a map array
// for separate bit planes of 8 bit pixels
//wi, hi  size of the input image in pixels
//wo, ho  size of the output image in pixels
//*vhs	pointer to input image
//*izs	pointer to output image
//bgc	the background color, when the required pixel is outside
//of the input image
void remap(int wi, int hi, int wo, int ho, unsigned char *vhs, unsigned char *izs, float *map, unsigned char bgc, interpp interp);

//-----------------------------------------------------------
//the same as above, but for packed color with 4 byte pixels
void remap32(int wi, int hi, int wo, int ho, unsigned char *vhs, unsigned char *izs, float *map, __u32 bgc, interpp interp);



//**********************************************************
//interpolating functions
//*sl	pointer to input image
//w,h	size of input image in pixels
//x,y	location of the interpolated point, in pixels
//*v	function output (interpolated value)

//-------------------------------------------------------
//nearest neighbor, with debugging printout
int interpNNpr_b(unsigned char *sl, int w, int h, float x, float y, unsigned char *v);

//--------------------------------------------------------
//nearest neigbor
int interpNN_b(unsigned char *sl, int w, int h, float x, float y, unsigned char *v);

//------------------------------------------------------
//bilinear
int interpBL_b(unsigned char *sl, int w, int h, float x, float y, unsigned char *v);

//---------------------------------------------------
//bicubic smooth
int interpBC_b(unsigned char *sl, int w, int h, float x, float y, unsigned char *v);

//-----------------------------------------------------
//bicubic sharp
int interpBC2_b(unsigned char *sl, int w, int h, float x, float y, unsigned char *v);

//---------------------------------------------------
//spline 4x4
int interpSP4_b(unsigned char *sl, int w, int h, float x, float y, unsigned char *v);

//-----------------------------------------------------
//spline 6x6
int interpSP6_b(unsigned char *sl, int w, int h, float x, float y, unsigned char *v);

//-------------------------------------------------------
//Lanczos 16x16
int interpSC16_b(unsigned char *sl, int w, int h, float x, float y, unsigned char *v);



//**********************************************************
//Following are the same interpolating functions as above,
//for packed color with 4 byte pixels
int interpNN_b32(unsigned char *sl, int w, int h, float x, float y, unsigned char *v);

//------------------------------------------------------
//bilinear
int interpBL_b32(unsigned char *sl, int w, int h, float x, float y, unsigned char *v);

//---------------------------------------------------
//bicubic smooth
int interpBC_b32(unsigned char *sl, int w, int h, float x, float y, unsigned char *v);

//-----------------------------------------------------
//bicubic sharp
int interpBC2_b32(unsigned char *sl, int w, int h, float x, float y, unsigned char *v);

//---------------------------------------------------
//spline 4x4
int interpSP4_b32(unsigned char *sl, int w, int h, float x, float y, unsigned char *v);

//-----------------------------------------------------
//spline 6x6
int interpSP6_b32(unsigned char *sl, int w, int h, float x, float y, unsigned char *v);

//-------------------------------------------------------
//Lanczos 16x16
int interpSC16_b32(unsigned char *sl, int w, int h, float x, float y, unsigned char *v);


